@@ -26,7 +26,8 @@ from datablender.base import (
     AsyncConnection,
     AsyncDataConfiguration,
     AsyncRequest,
-    DataSets
+    DataSets,
+    FileServer
 )
 
 from datablender.database import (
