@@ -35,7 +35,7 @@ from scipy.stats import iqr
 import numpy.core.defchararray as np_f
 from sklearn.model_selection import train_test_split
 
-from datablender.base.file import File
+from datablender.base.file import File,AsyncFile
 from datablender.base import getFunction
 from datablender.base.connection import Connection
 from datablender.base.queryBuilder import QueryBuilder
@@ -442,8 +442,8 @@ class Data:
                 )
         elif export_format == 'json':
             return [self.formatValue(a) for a in series] 
-        
-        return series
+        else:
+            return series
 
     def exportGeojson(
         self,
@@ -1671,7 +1671,7 @@ class Data:
         column_name:str=None,
         buffer = None,
         buffer_format = None,
-        file:File = None,
+        file:AsyncFile = None,
         *args,
         **kwargs
     ) -> None:
@@ -1685,40 +1685,65 @@ class Data:
         else:
             if not file:
                 if directory_name is None:
-                    directory_name = os.path.join(
-                        os.getenv(
-                            'DATA_DIRECTORY',
-                            os.getcwd()
-                        ),
-                        'temporary'
-                    )
+
+                    if "DATA_DIRECTORY" in os.environ:
+                        directory_name = os.path.join(
+                            os.getenv(
+                                'DATA_DIRECTORY',
+                                #os.getcwd()
+                            ),
+                            'temporary'
+                        )
+                    else:
+                        directory_name =os.path.join(
+                            os.getenv('FILE_SERVER_GEOGRAPHY_DIRECTORY'),
+                            'database' if self.schema_name else 'files',
+                        )
 
                 if file_name is None:
                     if column_name is not None:
-                        file_name = '{}_{}.{}'.format(
-                            self.name,
-                            column_name,
-                            extension
-                        )
+                        if self.schema_name:
+                            file_name = '{}.{}.{}.{}'.format(
+                                self.schema_name,
+                                self.name,
+                                column_name,
+                                extension
+                            )
+                        else :
+                            file_name = '{}.{}.{}'.format(
+                                self.name,
+                                column_name,
+                                extension
+                            )
                     elif self.name is not None:
                         file_name = '{}.{}'.format(
                             self.name,
                             extension
                         )
 
-                file = File(
+
+                file = AsyncFile(
                     directory_name,
                     file_name 
                 )
+
+            print(file.path)
             
-            file.write(
+            await file.write(
                 self.export(extension,column_name) if extension in ['geojson','json'] else self.frame,
                 *args,
                 **kwargs
             )
 
         if column_name is not None:
-            return 'http://localhost:8003/{}'.format(file_name)
+            return 'http://{}:{}/{}'.format(
+                os.getenv('FILE_SERVER_GEOGRAPHY_HOST','localhost'),
+                os.getenv('FILE_SERVER_GEOGRAPHY_PORT','8003'),
+                os.path.join(
+                    'database' if self.schema_name else 'files',
+                    file.name
+                )
+            )
       
     async def read(
         self,
@@ -1769,6 +1794,7 @@ class Data:
                     column_name = column_name
                 )
             }
+            print(meta_column)
         
         elif action_name == 'describe':
             meta_column = {
