@@ -8,6 +8,7 @@ import copy
 import shutil
 import datetime
 import asyncio
+import uuid
 
 from datablender.base.file import File
 from datablender.base.request import Request,AsyncRequest
@@ -400,7 +401,9 @@ class AsyncDataConfiguration:
                     )
                 
                 if self.request.session is None:
-                    await self.request.setSession()
+                    await self.request.setSession(
+                        verify_ssl=False
+                    )
 
             else:
                 self.directory_name = self.directory_name if self.directory_name else os.getenv(
@@ -462,6 +465,8 @@ class AsyncDataConfiguration:
         for attribute in ['size','sizes']:
             configuration.pop(attribute,None)
     
+        configuration['id'] = str(uuid.uuid4())
+
         if self.storage_type=='server':
 
             self.request.reset()
@@ -474,14 +479,10 @@ class AsyncDataConfiguration:
                 
                 assert response.status == 201
                 element = await response.json()
-                configuration['id'] = element['id']
+                #configuration['id'] = element['id']
 
         elif self.storage_type == 'file':
 
-            configuration['id'] = getNextID(
-                self.configuration_file.content[element_type]
-            )
-
             self.configuration_file.content[element_type].append(
                 configuration
             )
