@@ -384,34 +384,35 @@ class AsyncDataEventsTable:
         informations:dict,
         **kwargs
     ):
+        if hasattr(self,'elements_name') and self.exists:
         
-        event_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+            event_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
 
-        await self.query_builder.insert(
-            'data_events'
-        ).columns(
-            [
-               column['name'] for column in self.columns
-               if column['name'] != 'id'
-            ]
-        ).values(
-            [
-                event_name,
-                element_type,
-                element_configuration.get('id',None),
-                element_configuration.get('name',None),
-                element_configuration.get('schema_name',None),
-                event_time,
-                duration,
-                action_name,
-                action_time if action_name else event_time,
-                action_element_type if action_name else None,
-                action_element_configuration.get('id',None) if action_name else None,
-                action_element_configuration.get('name',None) if action_name else None,
-                action_element_configuration.get('schema_name',None) if action_name else None,
-                json.dumps(informations)
-            ]
-        ).built().asyncExecute(**kwargs)
+            await self.query_builder.insert(
+                'data_events'
+            ).columns(
+                [
+                column['name'] for column in self.columns
+                if column['name'] != 'id'
+                ]
+            ).values(
+                [
+                    event_name,
+                    element_type,
+                    element_configuration.get('id',None),
+                    element_configuration.get('name',None),
+                    element_configuration.get('schema_name',None),
+                    event_time,
+                    duration,
+                    action_name,
+                    action_time if action_name else event_time,
+                    action_element_type if action_name else None,
+                    action_element_configuration.get('id',None) if action_name else None,
+                    action_element_configuration.get('name',None) if action_name else None,
+                    action_element_configuration.get('schema_name',None) if action_name else None,
+                    json.dumps(informations)
+                ]
+            ).built().asyncExecute(**kwargs)
 
 class AsyncDataEventsView:
     """description
