@@ -11,6 +11,7 @@ import datetime
 
 from sqlalchemy.sql import text
 
+from datablender.base.configuration import configuration
 from datablender.base.file import File
 from datablender.base.connection import Connection,AsyncConnection
 
@@ -46,8 +47,7 @@ class QueryBuilder:
     """
     def __init__(
         self,
-        connection:Union[AsyncConnection,Connection],
-        config = None
+        connection:Union[AsyncConnection,Connection]
     ):
         """Initiate the query builder.
 
@@ -61,14 +61,7 @@ class QueryBuilder:
             '..',
             'postgresql'
         )
-        self.config = config if config else File(
-            path = os.path.join(
-                os.path.dirname(__file__),
-                '..',
-                'configuration.json'
-            )
-        ).read().content
-
+        self.config = configuration
         self.vocabulary=self.config['vocabulary']
         self.generic_constraints = self.config['generic_constraints']
 
