@@ -914,7 +914,9 @@ class AsyncDataProcess(AsyncDataElement):
             host = '10.4.0.13',
             loop=self.loop
         )
-        await request.setSession()
+        await request.setSession(
+            verify_ssl=False
+        )
         request.reset()
         request.addElement('area')
 
