@@ -20,6 +20,7 @@ from datablender.base import (
     DataConfiguration,
     Connection,
     File,
+    AsyncFile,
     DataLogging,
     Data,
     getFunction,
@@ -27,11 +28,13 @@ from datablender.base import (
     Bot,
     DataElement,
     Directory,
+    AsyncDirectory,
     AsyncRequest,
     AsyncDataLogging,
     AsyncConnection,
     AsyncDataConfiguration,
-    AsyncDataElement
+    AsyncDataElement,
+    FileServer
 )
 from datablender.database import (
     AsyncTable,
@@ -43,7 +46,7 @@ from datablender.data.directoryElementController import DirectoryElementControll
 from datablender.data.dataVersion import AsyncDataVersion
 from datablender.data.filesTable import AsyncFilesTable
 
-class AsyncRawDataFile(File):
+class AsyncRawDataFile(AsyncFile):
     """A file containing raw data from the source.
 
     Attributes:
@@ -77,7 +80,7 @@ class AsyncRawDataFile(File):
         rename_parameters:dict={},
         directory_name_setter:dict={},
         data_logging:AsyncDataLogging = None,
-        file_server:SMBConnection = None
+        file_server:FileServer = None
     ):
         """Initiate the raw data file.
 
@@ -259,7 +262,6 @@ class AsyncRawDataFile(File):
         )
 
         try:
-
             self.request = AsyncRequest(
                 self.is_secure,
                 self.host,
@@ -273,7 +275,7 @@ class AsyncRawDataFile(File):
             self.request.addElement(
                 self.name
             ) if self.file_included else None
-            
+
             async with self.request.session.get(
                 self.request.url,
                 allow_redirects=True
@@ -287,11 +289,11 @@ class AsyncRawDataFile(File):
 
                     if (
                         (
-                            Directory(self.directory_name,self.file_server).checkIfExists(self.name)
+                            AsyncDirectory(self.directory_name,self.file_server).checkIfExists(self.name)
                             and not self.rename_parameters
                         ) or (
                             self.rename_parameters
-                            and Directory(self.directory_name,self.file_server).checkIfExists(self.getNewName(
+                            and AsyncDirectory(self.directory_name,self.file_server).checkIfExists(self.getNewName(
                                 **self.rename_parameters
                             ))
                         )
@@ -300,11 +302,11 @@ class AsyncRawDataFile(File):
 
                 if(
                     (
-                        Directory(self.directory_name,self.file_server).checkIfExists(self.name)
+                        AsyncDirectory(self.directory_name,self.file_server).checkIfExists(self.name)
                         and not self.rename_parameters
                     ) or (
                         self.rename_parameters
-                        and Directory(self.directory_name,self.file_server).checkIfExists(self.getNewName(
+                        and AsyncDirectory(self.directory_name,self.file_server).checkIfExists(self.getNewName(
                             **self.rename_parameters
                         )+'.'+self.extension)
                     )
@@ -312,7 +314,7 @@ class AsyncRawDataFile(File):
                     return
 
                 chunk = await response.content.read()
-                self.write(chunk)
+                await self.write(chunk)
 
             if self.rename_parameters:
                 self.rename(
@@ -333,7 +335,7 @@ class AsyncRawDataFile(File):
         elements:List[dict],
         directory_name:str
     ) -> None:
-    
+        
         if (
             self.name is not None
             and os.path.abspath(
@@ -381,7 +383,7 @@ class AsyncDataFetcher:
         data_logging:AsyncDataLogging=None,
         loop =None,
         actions:List[dict] = [],
-        file_server:SMBConnection = None,
+        file_server:FileServer = None,
         schema_name:str =None
     ):
 
@@ -409,6 +411,13 @@ class AsyncDataFetcher:
         self.raw_data= None
 
         self.access_token = None
+        self.request = AsyncRequest(
+            self.is_secure,
+            self.host,
+            self.port,
+            self.domain_name,
+            self.loop
+        )
 
     @property
     def configuration(self) -> None:
@@ -423,7 +432,8 @@ class AsyncDataFetcher:
             'directory_name_setter': self.directory_name_setter,
             'bot_actions': self.bot_actions,
             'files': self.files,
-            'request_params':self.request_params
+            'request_params':self.request_params,
+            'actions':self.actions
         }
 
     def setRawFiles(
@@ -508,7 +518,12 @@ class AsyncDataFetcher:
                 )
 
             elif self.actions:
+                self.raw_files = []
+                await self.request.setSession(
+                    verify_ssl=False
+                )
                 await self.executeActions(is_multi_fetcher)
+                await self.request.close()
 
         except Exception as error:
             await self.data_logging.logEvent(
@@ -585,13 +600,16 @@ class AsyncDataFetcher:
         self,
         is_multi_fetcher:bool =False
     ) -> None:
-        for action in self.actions:
+        for action in copy.deepcopy(self.actions):
             await getattr(self,action.pop('name'))(
                 is_multi_fetcher=is_multi_fetcher,
                 **action
             )
 
-    async def setToken(self,is_multi_fetcher) -> None:
+    async def setToken(
+        self,
+        **kwargs
+    ) -> None:
         ## Token
         token_file = File(
             os.getenv('code_directory'),
@@ -614,133 +632,124 @@ class AsyncDataFetcher:
             refresh= True
         
         if refresh:
-            async with aiohttp.ClientSession(loop=self.loop) as session:
-                token_request = AsyncRequest(
-                    self.is_secure,
-                    self.host,
-                    self.port,
-                    self.domain_name,
-                    session=session
-                )
-                token_request.addElement('v1')
-                token_request.addElement('tokens')
+   
+            self.request.addElement('v1')
+            self.request.addElement('tokens')
 
-                async with token_request.session.post(
-                    token_request.url,
-                    json = {
-                        "refresh_token": os.getenv('token')
-                    }
-                ) as response:
-                    assert response.status == 200
-                    self.access_token = await response.json()
-                    token_file.write(self.access_token)
+            async with self.request.session.post(
+                self.request.url,
+                json = {
+                    "refresh_token": os.getenv('token')
+                }
+            ) as response:
+                assert response.status == 200
+                self.access_token = await response.json()
+                token_file.write(self.access_token)
+
+            self.request.reset()
 
     async def get(
         self,
-        url_elements:List[str],
+        url_elements:List[Dict[str,str]],
         params:dict = {},
         to_get:str = None,
         list_element:dict = None,
         date:str =None,
-        is_multi_fetcher:bool = False
+        is_multi_fetcher:bool = False,
+        **kwargs
     ) -> None:
         # List of feeds     
-        async with aiohttp.ClientSession(loop=self.loop) as session:
-            request = AsyncRequest(
-                self.is_secure,
-                self.host,
-                self.port,
-                self.domain_name,
-                session=session
+
+        for element in url_elements:
+            self.request.addElement(
+                list_element[element.get('key')]
+                if 'key' in element
+                else element.get('element')
             )
+        
+        async with self.request.session.get(
+            self.request.url,
+            headers = {"Authorization": "Bearer {}".format(self.access_token.get('access_token'))} if self.access_token else {},
+            params=params
+        ) as response:
+            assert response.status == 200
+            response_json:dict = await response.json()
 
-            for element in url_elements:
-                if isinstance(element,dict):
-                    element = list_element[element.get('key')]
-                    
-                request.addElement(element)
+            if to_get:
+                setattr(
+                    self,
+                    to_get,
+                    response_json.get(to_get)
+                )
 
-            async with request.session.get(
-                request.url,
-                headers = {"Authorization": "Bearer {}".format(self.access_token.get('access_token'))} if self.access_token else {},
-                params=params
-            ) as response:
-                assert response.status == 200
-                response_json:dict = await response.json()
-
-                if to_get:
-                    setattr(
-                        self,
-                        to_get,
-                        response_json.get(to_get)
-                    )
-
-                elif list_element and date:
-                    feeds:List[dict] = []
-                    date = datetime.datetime.strptime(date, "%Y-%m-%d").date()
-                    for feed in response_json:
-                        # 2025-01-09T00:22:56.418514Z
-                        feeds.append({
-                            **feed,
-                            'timedelta':abs(
-                                date-datetime.datetime.strptime(
-                                    feed['downloaded_at'],
-                                    "%Y-%m-%dT%H:%M:%S.%fZ"
-                                ).date()
-                            )
-                        })
-
-                    timedelta_min = min([f.get('timedelta') for f in feeds])
-                    
-                    self.raw_files.append(
-                        AsyncRawDataFile(
-                            self.is_secure,
-                            self.domain_name,
-                            self.host,
-                            self.port,
-                            copy.deepcopy(next((
-                                f.get('hosted_url') for f in feeds
-                                if timedelta_min == f.get('timedelta')
-                            ))),
-                            directory_name=self.directory_name,
-                            downloading_name = self.downloading_name,
-                            rename_parameters = self.rename_parameters,
-                            directory_name_setter=self.directory_name_setter,
-                            data_logging=self.data_logging
+            elif list_element and date:
+                feeds:List[dict] = []
+                date = datetime.datetime.strptime(date, "%Y-%m-%d").date()
+                for feed in response_json:
+                    # 2025-01-09T00:22:56.418514Z
+                    feeds.append({
+                        **feed,
+                        'timedelta':abs(
+                            date-datetime.datetime.strptime(
+                                feed['downloaded_at'],
+                                "%Y-%m-%dT%H:%M:%S.%fZ"
+                            ).date()
                         )
-                    )
+                    })
+
+                timedelta_min = min([f.get('timedelta') for f in feeds])
                 
-                elif list_element:
+                self.raw_files.append(
+                    AsyncRawDataFile(
+                        self.is_secure,
+                        self.domain_name,
+                        self.host,
+                        self.port,
+                        copy.deepcopy(next((
+                            f.get('hosted_url') for f in feeds
+                            if timedelta_min == f.get('timedelta')
+                        ))),
+                        directory_name=self.directory_name,
+                        downloading_name = self.downloading_name,
+                        rename_parameters = self.rename_parameters,
+                        directory_name_setter=self.directory_name_setter,
+                        data_logging=self.data_logging
+                    )
+                )
+            
+            elif list_element:
 
-                    if is_multi_fetcher and self.domain_name and '.' in self.domain_name:
-                        directory_name = os.path.join(
-                            self.directory_name,
-                            self.domain_name.split('.')[0]
-                        )
+                if is_multi_fetcher and self.domain_name and '.' in self.domain_name:
+                    directory_name = os.path.join(
+                        self.directory_name,
+                        self.domain_name.split('.')[0]
+                    )
 
-                        Directory(
-                            directory_name,
-                            self.file_server
-                        ).manage()
+                    await AsyncDirectory(
+                        directory_name,
+                        self.file_server
+                    ).manage()
 
-                    else:
-                        directory_name = self.directory_name
+                else:
+                    directory_name = self.directory_name
 
-                    self.raw_files.extend([
-                        AsyncRawDataFile(
-                            self.is_secure,
-                            self.domain_name,
-                            self.host,
-                            self.port,
-                            f['hosted_url'],
-                            directory_name=directory_name,
-                            downloading_name = self.downloading_name,
-                            rename_parameters = self.rename_parameters,
-                            directory_name_setter=self.directory_name_setter,
-                            data_logging=self.data_logging,
-                            file_server=self.file_server
-                        ) for f in response_json[0:1]
-                    ])
+                self.raw_files.extend([
+                    AsyncRawDataFile(
+                        self.is_secure,
+                        self.domain_name,
+                        self.host,
+                        self.port,
+                        f['hosted_url'],
+                        directory_name=directory_name,
+                        downloading_name = self.downloading_name,
+                        rename_parameters = self.rename_parameters,
+                        directory_name_setter=self.directory_name_setter,
+                        data_logging=self.data_logging,
+                        file_server=self.file_server
+                    ) for f in response_json[0:1]
+                ])
+        
+        self.request.reset()
 
     async def getFilesFromList(
         self,
@@ -750,7 +759,8 @@ class AsyncDataFetcher:
         list_from_filter:list = [],
         list_from_filter_key:str = None,
         date:str =None,
-        is_multi_fetcher:bool = False
+        is_multi_fetcher:bool = False,
+        **kwargs
     ) -> None:
 
         for list_element in getattr(self,list_from):
@@ -784,13 +794,13 @@ class AsyncDataSourceCore(AsyncDataElement):
         >>>     datablender.Connection()
         >>>     datablender.DataConfiguration(activate = True)
         >>> )
-        >>> print(data_source.schema)
+        >>> data_source.schema
         public
         >>> data_source.data_source_directory.name # Your current directory or env directory + '/data'
         >>> data_source.code_directory.name # Your current directory or env directory + '/code'
-        >>> print(data_source.file.name)
+        >>> data_source.file.name
         None
-        >>> print(data_source.tables[0].name)
+        >>> data_source.tables[0].name
         data
 
     """
@@ -817,7 +827,7 @@ class AsyncDataSourceCore(AsyncDataElement):
         event_server=None,
         views:List[Dict[str,any]] = [],
         loop =None,
-        file_server:SMBConnection = None
+        file_server:FileServer = None
     ):
         
         # structure in the data_config :
@@ -1036,9 +1046,9 @@ class AsyncDataSourceCore(AsyncDataElement):
         configuration:dict
     ) -> AsyncTable:
         
-        if 'id' in configuration and isinstance(configuration.get('id'),str):
-            configuration['name'] = configuration.get('id')
-            configuration['id'] = None
+        # if 'id' in configuration and isinstance(configuration.get('id'),str):
+        #     configuration['name'] = configuration.get('id')
+        #     configuration['id'] = None
         
         data_conditions = configuration.pop('data_conditions',{})
         
@@ -1087,44 +1097,15 @@ class AsyncDataSourceCore(AsyncDataElement):
         
         if self.directory_name is None:
 
-            data_host =os.getenv(
-                'DATA_HOST',
-                None
-            )
+            if self.file_server is not None:
 
-            if self.file_server:
-                pass
+                path = '/sources'
+                await AsyncDirectory(path,self.file_server).manage()
 
-            elif data_host:
+                path = os.path.join(path,self.schema_name)
+                await AsyncDirectory(path,self.file_server).manage()
 
-                self.file_server = SMBConnection(
-                    'jgelb',#os.getenv('username'),
-                    os.getenv('DATA_PASSWORD'),
-                    'tests',
-                    os.getenv('DATA_MACHINE_NAME')
-                )
-                self.file_server.connect(data_host,139)
-
-                if 'sources' not in [
-                    getattr(f,'filename') for f in self.file_server.listPath('data', "/")
-                ]:
-                    self.file_server.createDirectory('data','sources')
-                    
-                if self.schema_name not in [
-                    getattr(f,'filename') for f in self.file_server.listPath('data', "/sources")
-                ]:
-                    self.file_server.createDirectory('data',
-                        os.path.join(
-                            'sources',
-                            self.schema_name
-                        )
-                    )
-
-                self.directory_name = os.path.join(
-                    'sources',
-                    self.schema_name,
-                    self.name
-                )
+                self.directory_name = os.path.join(path,self.name)
 
             else:
 
@@ -1144,9 +1125,8 @@ class AsyncDataSourceCore(AsyncDataElement):
 
             for data_fetcher in self.data_fetchers:
                 data_fetcher.directory_name = self.directory_name
-                data_fetcher.file_server = self.file_server
 
-        Directory(
+        await AsyncDirectory(
             self.directory_name,
             self.file_server
         ).manage()
@@ -1189,23 +1169,15 @@ class AsyncDataSourceCore(AsyncDataElement):
             if 'views' in data_version or 'files' in data_version:
                 await self.manageDirectory()
             
-            self.data_version = AsyncDataVersion(
+            self.data_version = await AsyncDataVersion(
                 self.connection,
                 self.schema_name,
                 self.data_configuration,
                 active=True if data_version else False,
                 schema_id = self.schema_id,
                 database=self.database,
-                directory_name=self.directory_name,
                 **data_version
-            )
-
-            [
-                await table.manage() for table
-                in self.data_version.tables
-            ]
-
-            await self.data_version.getViews(
+            ).initiate(
                 self.directory_name,
                 self.file_server
             )
@@ -1221,7 +1193,8 @@ class AsyncDataSourceCore(AsyncDataElement):
                 directory_name=self.directory_name,
                 data_logging=self.data_logging,
                 loop=self.loop,
-                schema_name=self.schema_name
+                schema_name=self.schema_name,
+                file_server=self.file_server
             )
             for fetch_element in fetch
         ]
@@ -1375,7 +1348,8 @@ class AsyncDataSourceCore(AsyncDataElement):
         table_name:str=None,
         file_name:str=None,
         data_condition_id:int=None,
-        directory_name:str=None
+        directory_name:str=None,
+        **kwargs
     ) -> bool:
         """Check the data conditions using the table name and the file name.
 
@@ -1509,7 +1483,7 @@ class AsyncDataSourceCore(AsyncDataElement):
             )
             await self.manage('values')
             
-        sets = []
+        sets:List[Data] = []
         
         for table in self.tables:
             if (
@@ -1520,14 +1494,15 @@ class AsyncDataSourceCore(AsyncDataElement):
 
                 for data_conditions in table.data_conditions[self.name]:
                     data_set = copy.deepcopy(data)
-                    await self.dataTableActions(
+                    has_condition = await self.dataTableActions(
                         actions,
                         table,
                         data_conditions,
                         data=data_set,
+                        set = sets,
                         **kwargs
                     )
-                    if data_set:
+                    if data_set and has_condition:
                         frames.append(data_set.frame)
 
                 if frames:
@@ -1536,7 +1511,7 @@ class AsyncDataSourceCore(AsyncDataElement):
 
             else:
                 data_set = copy.deepcopy(data)
-                await self.dataTableActions(
+                has_condition = await self.dataTableActions(
                     actions,
                     table,
                     table.data_conditions[self.name] if isinstance(
@@ -1544,12 +1519,15 @@ class AsyncDataSourceCore(AsyncDataElement):
                         dict
                     ) else {},
                     data=data_set,
+                    set = sets,
                     **kwargs
                 )
 
-                sets.append(data_set)
+                if has_condition:
+                    sets.append(data_set)
 
-        return sets[0]
+        if len(sets) > 1:
+            return sets[0]
         
     async def dataTableActions(
         self,
@@ -1559,7 +1537,7 @@ class AsyncDataSourceCore(AsyncDataElement):
         file_name:str=None,
         directory_name:str=None,
         **kwargs
-    ) -> None:
+    ) -> bool:
         """Execute action for a table.
 
         Args:
@@ -1589,6 +1567,8 @@ class AsyncDataSourceCore(AsyncDataElement):
                     directory_name= directory_name,
                     **kwargs
                 )
+            return True
+        return False
 
     async def executeDataTableAction(
         self,
@@ -1822,30 +1802,37 @@ class AsyncDataSourceCore(AsyncDataElement):
                             values.assign(file_name = file_name)
                         ])
 
-                        file.write(file.data.frame)
+                        await file.write(file.data.frame)
                     else:
                         values = values.assign(file_name = file_name)
-                        file.write(values)
+                        await file.write(values)
                         file.data = Data(values)
 
                     return False  
             
-            {
-                value.setValue(
-                    file_name,
-                    main_files[0].get('directory_name') if main_files else directory_name,
-                    main_files[0].get('name') if main_files else None,
-                    data
-                ) 
-                for value in self.data_version.values
-                if self.checkDataConditions(
-                    value.data_conditions,
-                    None,
-                    file_name,
-                    None,
-                    main_files[0].get('directory_name') if main_files else directory_name
-                )
-            }
+            self.setDataVersionValues(
+                file_name = file_name,
+                directory_name = main_files[0].get('directory_name') if main_files else directory_name,
+                main_file_name = main_files[0].get('name') if main_files else None,
+                data = data
+            )
+
+            # {
+            #     value.setValue(
+            #         file_name,
+            #         main_files[0].get('directory_name') if main_files else directory_name,
+            #         main_files[0].get('name') if main_files else None,
+            #         data
+            #     ) 
+            #     for value in self.data_version.values
+            #     if self.checkDataConditions(
+            #         value.data_conditions,
+            #         None,
+            #         file_name,
+            #         None,
+            #         main_files[0].get('directory_name') if main_files else directory_name
+            #     )
+            # }
 
             if update_version:
                 return True
@@ -1932,40 +1919,23 @@ class AsyncDataSourceCore(AsyncDataElement):
                         data.frame[data_version_value.get('name')].dt.strftime('%Y-%m-%d') == data_version_value.get('value')
                     ]
 
-    def getTemporaryFile(
-        self,
-        name:str,
-        directory_name:str,
-        main_file_path:str = None
-    ) -> Dict[str, str]:
-        """Get the saved temporary file associated 
-        with the main file.
-
-        Args:
-            name (str): Main file name.
-            directory_name (str): Main file directory.
-            main_file_path (str): Main file path.
-
-        Returns:
-            Dict[str, str]: Temporary directory parameteres.
-        """
-        return next(
-            (
-                main_file for main_file
-                in self.temporary_main_files
-                if (
-                    main_file.get('name') == name
-                    and main_file.get(
-                        'directory_name'
-                    ) == directory_name
-                )
-            ),
-            None
-        )
-
     async def saveVersions(
         self,
         database:AsyncDatabase
     ) -> None:
         for view in self.data_version.views:
             await view.saveVersions(database)
+
+    def setDataVersionValues(
+        self,
+        **kwargs
+    ):
+        {
+            value.setValue(
+                **kwargs
+            ) 
+            for value in self.data_version.values
+            if self.checkDataConditions(
+                **kwargs
+            )
+        }
