@@ -8,16 +8,21 @@ import os
 import copy
 import tempfile
 import shutil
+from io import BytesIO
 from smb.SMBConnection import SMBConnection
 
 from datablender.base import (
     Directory,
+    AsyncDirectory,
     ZipFile_,
+    AsyncZipFile,
     DirectoryElement,
+    AsyncDirectoryElement,
     Connection,
     DataConfiguration,
     AsyncConnection,
-    AsyncDataConfiguration
+    AsyncDataConfiguration,
+    FileServer
 )
 from datablender.database import (
     Database,
@@ -746,7 +751,7 @@ class DataZipFile(ZipFile_):
             file.name for file in files if file.name is not None
         ] else 'old'
 
-class AsyncDataDirectory(Directory):
+class AsyncDataDirectory(AsyncDirectory):
     """Directory containing files with data.
 
     Attributes:
@@ -787,7 +792,7 @@ class AsyncDataDirectory(Directory):
         control:dict = {},
         data_version:dict = {},
         database:AsyncDatabase = None,
-        database_name:str= None,
+        database_name:str= None,    
         schema:Union[AsyncSchema,str] = 'public',
         table:Union[AsyncTable,str,list] = None,
         event_server = None,
@@ -858,22 +863,25 @@ class AsyncDataDirectory(Directory):
             )
         }
 
-    def setElements(self) -> None:
+    async def setElements(self) -> None:
         """Set the elements by initiatig every element with the Data Directory Element.
         """
         
+        await self.getElementsName()
+
         self.elements = [
-            AsyncDataDirectoryElement(
+            await AsyncDataDirectoryElement(
                 self.data_source_core,
                 name = element_name,
                 directory_name = self.name,
                 main_files = copy.deepcopy(self.main_files),
                 file_server=self.file_server,
                 is_temporary= self.is_temporary
-            ).control(
+            ).initiate(
                 self.elements_name
-            ) for element_name in self.elements_name
+            ) for element_name in self.elements_name 
         ]
+        self.elements = [e for e in self.elements if not e.is_temporary_for_zip(self.elements_name)]
 
     def orderElements(self) -> None:
         """List of the files in a directory, if there's a table list in config,
@@ -924,29 +932,37 @@ class AsyncDataDirectory(Directory):
     async def saveFiles(self, **kwargs) -> None:
         """Save files in directory.
         """
-        self.setElements()
+        await self.setElements()
         await self.manageElements()
         self.orderElements()
         
         for element in self.elements:
             await element.save(**kwargs)
 
-    def setElementStatus(
+    async def setElementStatus(
         self,
         files:List[AsyncRawDataFile],
         check_directory:bool = False
     ) -> List[RawDataFile]:
 
-        self.setElements()
+        await self.setElements()
+
+        ##################################
+        ### Current directory elements ###
 
         # For every elements in the directory, set the status
         [
-            element.setStatus(
+            await element.setStatus(
                 files,
                 check_directory
             )
             for element in self.elements
         ]
+        
+        #################
+        ### Raw files ###
+
+        # For every raw files, set the status
         [
             file.setStatus(
                 [
@@ -958,6 +974,8 @@ class AsyncDataDirectory(Directory):
             for file in files
         ]
 
+        ##########################################
+        ### Add raw files to directory elements ###
         self.raw_files = [
             file for file in files
             if (
@@ -970,7 +988,7 @@ class AsyncDataDirectory(Directory):
             ) 
         ]
 
-class AsyncDataDirectoryElement(DirectoryElement):
+class AsyncDataDirectoryElement(AsyncDirectoryElement):
     """Element of a directory containing data, which can be a DataFile, DataDirectory of a ZipFile_.
 
     Attributes:
@@ -993,13 +1011,15 @@ class AsyncDataDirectoryElement(DirectoryElement):
         directory_name:str = None,
         path:str = None,
         main_files:List[Dict[str,str]]= [],
-        file_server:SMBConnection = None,
+        file_server:FileServer = None,
         is_temporary:bool = False,
         **kwargs
     ):
         self.data_source_core = data_source_core
         self.main_files = main_files
+        self.file_server=file_server
 
+        # If the element is in a zip file!!!!
         if path or (name and directory_name):
             super(AsyncDataDirectoryElement,self).__init__(
                 name,
@@ -1017,9 +1037,26 @@ class AsyncDataDirectoryElement(DirectoryElement):
         self.file:AsyncDataFile = None
         self.directory:AsyncDataDirectory = None
         self.zip_file:AsyncDataZipFile = None
+    
+    async def initiate(
+        self,
+        elements_name:List[str] =[]
+    ) -> AsyncDataDirectoryElement:
+        # If the element is in a zip file!!!!
+        
+        if hasattr(self,'directory_name'):
+            await self.checkType()
+            self.setElement()      
 
-        if path or (name and directory_name):
-            self.setElement()
+            if self.zip_file is not None:
+                await self.zip_file.initiate()
+                self.zip_file.checkData()
+            elif self.file is not None:
+                await self.file.initiateDataFile()
+
+            self.control(elements_name)
+
+        return self
     
     @property
     def informations(self) -> dict:
@@ -1036,9 +1073,15 @@ class AsyncDataDirectoryElement(DirectoryElement):
         else:
             return {}
 
-    def setElement(self) -> None:
-        if self.is_directory:
+    def is_temporary_for_zip(
+        self,
+        directory_element_names:List[str]
+    ) -> dict:
+        return self.is_directory and self.name.lower()+'.zip' in [n.lower() for n in directory_element_names]
 
+    def setElement(self) -> None:
+
+        if self.is_directory:
             self.directory = AsyncDataDirectory(
                 self.path+'/',
                 main_files=copy.deepcopy(self.main_files),
@@ -1081,7 +1124,7 @@ class AsyncDataDirectoryElement(DirectoryElement):
             self.save_file = False
             self.save_zip_file = False
 
-            if self.is_directory and self.name.lower()+'.zip' not in [n.lower() for n in directory_element_names]:
+            if self.is_directory and not self.is_temporary_for_zip(directory_element_names):
                 self.save_directory = self.data_source_core.element_controller.control(
                     self.name,
                     'directory'
@@ -1130,8 +1173,8 @@ class AsyncDataDirectoryElement(DirectoryElement):
     async def get(
         self,
         extract_files:bool = False,
+        data_actions:List[str] = [],
         transformation:list = [],
-        add_data_version:bool = False,
         columns_actions:List[dict] =[],
         **kwargs
     ) -> dict:
@@ -1142,31 +1185,38 @@ class AsyncDataDirectoryElement(DirectoryElement):
 
                 main_file_config = copy.deepcopy(main_file_config)
                 main_file_path = main_file_config.pop('main_file_path')
-
+    
             # If it is a file
             if '.' in self.name:
 
                 # Check if the main file has a temporary directory
-                temporary_directory = self.data_source_core.getTemporaryFile(
-                    **main_file_config
+                # temporary_directory = self.data_source_core.getTemporaryFile(
+                #     **main_file_config
+                # )
+
+                temporary_directory = AsyncDirectory(
+                    os.path.join(
+                        main_file_config.get('directory_name'),
+                        main_file_config.get('name')[:-4]
+                    ),
+                    self.file_server,
+                    True
                 )
+                await temporary_directory.getParentDirectory()
 
                 # If it's a file and there is a temporary directory, 
                 # then initiate the data directory element
-                if temporary_directory:
+                if temporary_directory.exists:
                     
                     super(AsyncDataDirectoryElement,self).__init__(
                         self.name,
                         os.path.join(
-                            temporary_directory.get(
-                                'temporary_directory_name'
-                            ),
+                            temporary_directory.name,
                             main_file_path if main_file_path else ''
-                        )
+                        ),
+                        file_server=self.file_server
                     )
-
-                    self.setElement()
-                    self.control()
+                    await self.initiate()
                     
                 else:
                     return {}
@@ -1192,15 +1242,23 @@ class AsyncDataDirectoryElement(DirectoryElement):
         if self.zip_file and extract_files:
 
             # Check if the main file has a temporary directory
-            temporary_directory = self.data_source_core.getTemporaryFile(
-                self.name,
-                self.directory_name
-            )
+            # temporary_directory = self.data_source_core.getTemporaryFile(
+            #     self.name,
+            #     self.directory_name
+            # )
 
-            if temporary_directory:
-                self.zip_file.temporary_directory_name = temporary_directory.get(
-                    'temporary_directory_name'
-                )
+            temporary_directory = AsyncDirectory(
+                os.path.join(
+                    self.directory_name,
+                    self.name[:-4]
+                ),
+                self.file_server,
+                True
+            )
+            await temporary_directory.getParentDirectory()
+
+            if temporary_directory.exists:
+                self.zip_file.temporary_directory_name = temporary_directory.name
             else:
                 await self.zip_file.extractFiles()
                 
@@ -1211,11 +1269,12 @@ class AsyncDataDirectoryElement(DirectoryElement):
                 })
             
         elif self.file:
+            
             await self.file.extract(**kwargs)
             
-            if transformation or add_data_version:
+            if data_actions:
                 
-                self.data_source_core.data_version.setValues(
+                self.data_source_core.setDataVersionValues(
                     **(
                         self.main_files[0]
                         if self.main_files else self.file.informations
@@ -1225,7 +1284,7 @@ class AsyncDataDirectoryElement(DirectoryElement):
                 if self.data_source_core.tables:
 
                     self.file.data = await self.data_source_core.actionsTables(
-                        ['transform','version'] if add_data_version else ['transform'],
+                        data_actions,
                         self.file.data,
                         file_name = self.file.name,
                         directory_name = self.file.name,
@@ -1237,7 +1296,7 @@ class AsyncDataDirectoryElement(DirectoryElement):
                         file_name = self.file.name,
                         directory_name = self.file.directory_name
                     )
-                    if add_data_version:
+                    if 'verion' in data_actions:
                         self.data_source_core.updateData(
                             self.file.data,
                             self.file.name,
@@ -1250,19 +1309,19 @@ class AsyncDataDirectoryElement(DirectoryElement):
             ]
 
         elif self.directory:
-            self.directory.setElements()
+            await self.directory.setElements()
 
         check_directory = kwargs.pop('check_directory',False)
         for data_fetcher in self.data_source_core.data_fetchers:
-            self.setStatus(
+            await self.setStatus(
                 data_fetcher.raw_files,
                 check_directory
             )
-        
+
         self.data_source_core.data_logging.action_name = None
         return self.informations
 
-    def setStatus(
+    async def setStatus(
         self,
         files:List[AsyncRawDataFile],
         check_directory:bool = True
@@ -1272,7 +1331,7 @@ class AsyncDataDirectoryElement(DirectoryElement):
             self.file.setStatus(files)
         
         if self.directory and check_directory:
-            return self.directory.setElementStatus(
+            return await self.directory.setElementStatus(
                 files,
                 check_directory
             )    
@@ -1280,7 +1339,7 @@ class AsyncDataDirectoryElement(DirectoryElement):
         if self.zip_file:
             self.zip_file.setStatus(files)
 
-class AsyncDataZipFile(ZipFile_):
+class AsyncDataZipFile(AsyncZipFile):
     """Represent a zip file with data.
 
     Attributes:
@@ -1337,7 +1396,7 @@ class AsyncDataZipFile(ZipFile_):
         table:Union[AsyncTable,str,list] = None,
         event_server = None,
         actions:List[Dict[str,any]] = [],
-        file_server:SMBConnection = None,
+        file_server:FileServer = None,
         is_temporary:bool = False
     ):
         """Initiate the zip file.
@@ -1382,8 +1441,6 @@ class AsyncDataZipFile(ZipFile_):
             event_server=event_server,
             actions=actions
         )
-        
-        self.checkData()
 
         self.main_files = main_files
         self.elements = []
@@ -1411,7 +1468,8 @@ class AsyncDataZipFile(ZipFile_):
                 'main_files':main_files,
                 'control':self.data_source_core.element_controller.control(
                     element,
-                    'file' if '.' in element else 'directory'
+                    'file' if  element.endswith('/') else 'directory'
+
                 )
             } for element in list(set(
                 [
@@ -1446,11 +1504,13 @@ class AsyncDataZipFile(ZipFile_):
     
     async def extractFiles(self) -> None:
 
-        directory = Directory(
+        directory = AsyncDirectory(
             os.path.join(
                 self.directory_name,
                 self.name_
-            )
+            ),
+            self.file_server,
+            True
         )
 
         await self.data_source_core.data_logging.logEvent(
@@ -1465,13 +1525,41 @@ class AsyncDataZipFile(ZipFile_):
         try:
 
             self.temporary_directory_name = directory.name
-            
-            if not directory.exists:
-                directory.manage()
-                self.extractall(
-                    self.temporary_directory_name +"\\"
+
+            if self.file_server:
+
+                directory.parent_directory = await directory.file_server.getParentDirectory(
+                    self.share_name,
+                    directory.name
                 )
+                if not directory.exists:
+                    await directory.make()
+                    for member in self.namelist():
+                        path = os.path.join(
+                            directory.name,
+                            member
+                        )
+                        if  member.endswith('/'):
+                            await self.file_server.makeDirectory(
+                                'Data',
+                                path
+                            )
+                        
+                        else:
+                            await self.file_server.writeFile(
+                                member,
+                                BytesIO(self.read(member)),
+                                path
+                            )
+
+            else:
             
+                if not directory.exists:
+                    await directory.make()
+                    self.extractall(
+                        self.temporary_directory_name +"\\"
+                    )
+                
         except Exception as error:
             directory.delete()
 
@@ -1520,15 +1608,15 @@ class AsyncDataZipFile(ZipFile_):
                 'main_file_path':None
             })
             
-            await AsyncDataDirectory(
-                self.temporary_directory_name +"\\",
+            directory = AsyncDataDirectory(
+                self.temporary_directory_name +"/",
                 main_files = copy.deepcopy(self.main_files),
                 data_source_core=self.data_source_core,
                 is_temporary=True,
                 file_server=self.file_server
-            ).saveFiles(**kwargs)
-            
-            shutil.rmtree(self.temporary_directory_name)
+            )
+            await directory.saveFiles(**kwargs)
+            await directory.delete()
 
     def setStatus(
         self,
