@@ -16,11 +16,13 @@ from smb.SMBConnection import SMBConnection
 
 from datablender.base import (
     File,
+    AsyncFile,
     DataConfiguration,
     Connection,
     Data,
     AsyncConnection,
-    AsyncDataConfiguration
+    AsyncDataConfiguration,
+    FileServer
 )
 from datablender.database import (
     Table,
@@ -327,7 +329,7 @@ class DataFile(File):
             file.name for file in files if file.name is not None
         ] else 'old'
 
-class AsyncDataFile(File):
+class AsyncDataFile(AsyncFile):
     """A file contening data to go in a database.
 
     Attributes:
@@ -378,7 +380,7 @@ class AsyncDataFile(File):
         table:Union[AsyncTable,str,list] = None,
         event_server = None,
         actions:List[Dict[str,any]] = [],
-        file_server:SMBConnection = None,
+        file_server:FileServer = None,
         is_temporary:bool = False
     ):
         super(AsyncDataFile,self).__init__(
@@ -390,12 +392,6 @@ class AsyncDataFile(File):
         )
         
         self.main_files = main_files
-        
-        self.data_modification_time = File(
-            self.main_files[0].get('directory_name'),
-            self.main_files[0].get('name'),
-            file_server=self.file_server
-        ).modification_time if self.main_files else self.modification_time
 
         self.data_source_core = data_source_core if data_source_core else AsyncDataSourceCore(
             connection,
@@ -429,6 +425,23 @@ class AsyncDataFile(File):
         self.save_data = True
         self.download_status = None
     
+    async def initiateDataFile(
+        self
+    ) -> AsyncDataFile:
+        
+        await self.initiate()
+
+        if self.main_files:
+                    
+            main_file = await AsyncFile(
+                self.main_files[0].get('directory_name'),
+                self.main_files[0].get('name'),
+                file_server=self.file_server
+            ).initiate()
+            self.data_modification_time = main_file.modification_time 
+        else: 
+            self.data_modification_time =self.modification_time
+
     @property
     def path_index(self) -> str:
         if self.main_files:
@@ -530,17 +543,17 @@ class AsyncDataFile(File):
 
             loaded_rows = kwargs.pop('loaded_rows',kwargs.get('nrows',None))
             transform_columns = kwargs.pop('columns',[])
-
+            await self.read(
+                *args,
+                **{
+                    **self.data_source_core.getExtractArguments(
+                        self.name
+                    ),
+                    **kwargs
+                }
+            )
             self.data = Data(
-                self.read(
-                    *args,
-                    **{
-                        **self.data_source_core.getExtractArguments(
-                            self.name
-                        ),
-                        **kwargs
-                    }
-                ).content,
+                self.content,
                 self.name,
                 self.directory_name,
                 name = self.name_,
@@ -579,10 +592,10 @@ class AsyncDataFile(File):
             data_version_value,
             update_version
         )
-        
+                
         if self.save_data:
             await self.checkInDatabase(import_mode)
-
+        
         if self.save_data:
             await self.extract()
 
