@@ -12,7 +12,8 @@ from datablender.base import (
     Connection,
     Data,
     AsyncDataConfiguration,
-    AsyncConnection
+    AsyncConnection,
+    FileServer
 )
 from datablender.database import Database, AsyncDatabase
 from datablender.data.dataSourceCore import  DataSourceCore
@@ -313,7 +314,8 @@ class AsyncDataSource:
         tables:Union[str,List[str],List[dict]] = None,
         event_server = None,
         views:List[Dict[str,any]] = [],
-        loop =None
+        loop =None,
+        file_server:FileServer = None
     ) -> None:
         
         self.name = name
@@ -343,7 +345,8 @@ class AsyncDataSource:
             tables,
             event_server=event_server,
             views = views,
-            loop=loop
+            loop=loop,
+            file_server=file_server
         )
 
     async def initiate(self) -> AsyncDataSource:
@@ -415,12 +418,12 @@ class AsyncDataSource:
         ):
             await self.core.manageDirectory()
             directory_element['path'] = self.core.directory_name
-        
-        self.directory_element = AsyncDataDirectoryElement(
+
+        self.directory_element = await AsyncDataDirectoryElement(
             self.core,
             file_server=self.core.file_server,
             **directory_element
-        ).control()
+        ).initiate()
 
         return self
     
@@ -448,7 +451,7 @@ class AsyncDataSource:
         return await self.getDataDirectoryElement(
             directory_element,
             relative_path,
-            check_directory = len(self.core.data_fetchers) > 1,
+            check_directory = True,
             **kwargs
         )
     
