@@ -10,6 +10,8 @@ import re
 import copy
 import datetime
 
+from smb.SMBConnection import SMBConnection
+
 from datablender.base import (
     DataConfiguration,
     Connection,
@@ -740,24 +742,70 @@ class DataSourceCore(DataElement):
     def manageDirectory(self) -> None:  
         
         if self.directory_name is None:
-            self.directory_name = os.path.join(
-                os.getenv(
-                    'DATA_DIRECTORY',
-                    os.getcwd()
-                ),
-                self.schema_name,
-                self.name
+
+            data_host =os.getenv(
+                'DATA_HOST',
+                None
             )
 
+            if self.file_server:
+                pass
+
+            elif data_host:
+
+                self.file_server = SMBConnection(
+                    'jgelb',#os.getenv('username'),
+                    os.getenv('DATA_PASSWORD'),
+                    'tests',
+                    os.getenv('DATA_MACHINE_NAME')
+                )
+                self.file_server.connect(data_host,139)
+
+                if 'sources' not in [
+                    getattr(f,'filename') for f in self.file_server.listPath('data', "/")
+                ]:
+                    self.file_server.createDirectory('data','sources')
+                    
+                if self.schema_name not in [
+                    getattr(f,'filename') for f in self.file_server.listPath('data', "/sources")
+                ]:
+                    self.file_server.createDirectory('data',
+                        os.path.join(
+                            'sources',
+                            self.schema_name
+                        )
+                    )
+
+                self.directory_name = os.path.join(
+                    'sources',
+                    self.schema_name,
+                    self.name
+                )
+
+            else:
+
+                self.directory_name = os.path.join(
+                    os.getenv(
+                        'DATA_DIRECTORY',
+                        os.getcwd()
+                    ),
+                    'sources',
+                    self.schema_name,
+                    self.name
+                )
+
             self.manage(new_configuration={
                 'directory_name':self.directory_name
             })
 
             for data_fetcher in self.data_fetchers:
                 data_fetcher.directory_name = self.directory_name
+                data_fetcher.file_server = self.file_server
 
-        if not os.path.isdir(self.directory_name):
-            Directory(self.directory_name).make()
+        Directory(
+            self.directory_name,
+            self.file_server
+        ).manage()
         
         return self.directory_name
 
