@@ -608,7 +608,20 @@ class AsyncDatabase(AsyncDatabaseElement):
         status:str = 'default',
         data_configuration:AsyncDataConfiguration = None,
         acticvate_data_config:bool = False,
-        grants:list = [],
+        grants:list = [
+            {
+                'privilege': 'temporary',
+                'user_name': 'postgres'
+            },
+            {
+                'privilege': 'create',
+                'user_name': 'postgres'
+            },
+            {
+                'privilege': 'connect',
+                'user_name': 'postgres'
+            }
+        ],
         event_server=None,
         owner_id:int = None
     ):
