@@ -677,17 +677,17 @@ class AsyncElementSQL(AsyncDataElement):
         new_value:Union[str,list,List[dict]]
     ) -> None:
         
-        old_value = self.setCompareValue(
+        old_value = copy.deepcopy(self.setCompareValue(
             getattr(self,attribute_name)
-        )
+        ))
         if isinstance(new_value,list) and isinstance(old_value,list):
             if all([
                 isinstance(sub_value,dict)
                 for sub_value in new_value
             ]):
-                if old_value != self.setCompareValue(new_value):
+                if old_value != self.setCompareValue(copy.deepcopy(new_value)):
                     new_values = copy.deepcopy(new_value)
-                    new_value = copy.deepcopy(old_value)
+                    new_value = old_value
 
                     for new_value_ in new_values:
                         new_value_ = self.setCompareValue(new_value_)
