@@ -73,7 +73,11 @@ class Schema(DatabaseElement):
         data_configuration:DataConfiguration = None,
         acticvate_data_config:bool = False,
         id:int = None,
-        content:dict = None,
+        content:dict = {
+          "name": None,
+          "description":None,
+          "content_elements": []
+        },
         schema_type:str = 'source',
         grants:List[dict] = [
             {
