@@ -895,7 +895,11 @@ class AsyncTable(AsyncSchemaElement):
         constraints:List[dict] = [],
         indexes:List[dict] = [],
         data:Data = None,
-        content:dict = None,
+        content:dict = {
+          "name": None,
+          "description":None,
+          "content_elements": []
+        },
         grants:List[dict] = None,
         size:int = 0,
         event_server = None,
@@ -1502,6 +1506,7 @@ class AsyncTable(AsyncSchemaElement):
                 geometry_saved_type='text',
                 name=self.name,
                 loaded_rows = loaded_rows,
+                schema_name=self.schema_name
             )
         
         return self
