@@ -76,7 +76,11 @@ class View(SchemaElement):
         schema_name:str = 'public',
         is_materialized:bool = False,
         owner:str = 'postgres',
-        content:list = None,
+        content:dict = {
+          "name": None,
+          "description":None,
+          "content_elements": []
+        },
         indexes:List[dict] = [],
         status:str = 'default',
         data_configuration:DataConfiguration = None,
@@ -655,7 +659,8 @@ class AsyncView(AsyncSchemaElement):
                 columns = self.columns,
                 geometry_saved_type='text',
                 name = self.name,
-                loaded_rows= loaded_rows
+                loaded_rows= loaded_rows,
+                schema_name=self.schema_name
             )
         else:
             self.data = Data(
@@ -672,7 +677,8 @@ class AsyncView(AsyncSchemaElement):
                 columns=[],
                 geometry_saved_type='text',
                 name = self.name,
-                loaded_rows= loaded_rows
+                loaded_rows= loaded_rows,
+                schema_name=self.schema_name
             )
 
         return self
